
package XML;
@ISA = qw( _SGML );

####################################
# Class Variables

# Emacs state
$InputStateKeyMap{$main::KeyCode{"^T"}}=\&XML::FindNextOpeningTag;
$InputStateKeyMap{$main::KeyCode{"^P"}}=\&XML::FindPrevOpeningTag;
$InputStateKeyMap{$main::KeyCode{"^W"}}=\&XML::GetMultiKey;

# Multiple Keystrokes Map
$MultiKeyMap{$main::KeyCode{"^M"}}=\&XML::FindMatchingTag;
$MultiKeyMap{$main::KeyCode{"^U"}}=\&XML::FindLastUnmatchedTag;
$MultiKeyMap{$main::KeyCode{"^B"}}=\&XML::BalanceLastUnmatchedTag;
$MultiKeyMap{$main::KeyCode{"^A"}}=\&XML::BalanceAllUnmatchedTags;

# XML doesn't support missing end tags
@ClosingTagOptional = qw( );

####################################
# Class Methods

####################################
# Instance Methods

sub _localize {
	#_DebugLog("Fund _localize\n");
	my($sub) = shift;
        my($this) = shift;
	# Setup some convenient aliases
	local(*curcol)  = \$this->{'curcol'};
	local(*trycol)  = \$this->{"trycol"};
        local(*curline) = \$this->{'curline'};
	local(*buffer)  =  $this->{'buffer'};
	local(*self)    = \$this;
        return &$sub(@_);
}

sub Initialize {
        my($self) = shift;

	$self->SUPER::Initialize;
	&_XML($self);
        return $self;
}

sub XMLMode {
        my($self) = shift;

        bless $self; # change the inheritance
	&_XML($self);
        return $self;
}
sub _XML {
	my($self) = shift;
	$self->{"curdisp"} = [];
        $self->{"mode desc"} = "XML";
	$self->{"input state keymap"} =
		{%Fundamental::InputStateKeyMap,%InputStateKeyMap};
	$self->ChangeToInputState;
}
sub ShowHelp {
	my($self) = shift;
	$self->SUPER::ShowHelp;
}
sub Help {
	return "XML HELP",[ split "\n", <<HELP_TEXT

                         XML Mode Help

   For additional help, see the help file for the Fundamental mode.

   INPUT    COMMAND  Description
   -------- -------- ----------------------------------------------------------
   ^T                Find Next Opening Tag
   ^P                Find Prev Opening Tag
   ^W^M              Find Matching Tag
                     Find Next Opening Tag at same level
                     Find Prev Opening Tag at same level
                     Add Tag (on > add \\n</tag>)
                     Delete from Opening Tag to Closing Tag
   ^W^U              Find Last Unmatched Opening Tag
   ^W^B              Add Mate To Last Unmatched Opening Tag
   ^W^A              Add Mate To All Unmatched Opening Tags
                     Indent by tag structure from Opening Tag to Closing Tag

            :FundamentalMode
                     Switch to Fundamental Mode

HELP_TEXT
];
}

###################
# MultiKey Functions for Input State

sub GetMultiKey {
	my($self) = shift;

	$self->ShowMsg("^W");
	$key=$self->GetKey(1);
	$self->DispatchKey($key,\%MultiKeyMap);
}

###################
#XML Movement

sub FindNextOpeningTag {
	my($self) = shift;
	return $self->SUPER::FindNextOpeningTag;
}
sub FindPrevOpeningTag {
	my($self) = shift;
	return $self->SUPER::FindPrevOpeningTag;
}
sub FindMatchingTag {
	my($self) = shift;
	return $self->SUPER::FindMatchingTag(@ClosingTagOptional);
}
sub FindLastUnmatchedTag {
	my($self) = shift;
	return $self->SUPER::FindLastUnmatchedTag(@ClosingTagOptional);
}
sub BalanceLastUnmatchedTag {
	my($self) = shift;
	return $self->SUPER::BalanceLastUnmatchedTag(@ClosingTagOptional);
}
sub BalanceAllUnmatchedTags {
	my($self) = shift;
	return $self->SUPER::BalanceAllUnmatchedTags(@ClosingTagOptional);
}

1;
